@@ -1,8 +1,9 @@
-
 import commpy as comm
 import numpy as np
 import soundfile as sf
 import bitstring as bs
+import requests #
+import os.path #
 from scipy.signal import max_len_seq
 from scipy.signal import resample
 from scipy.signal import filtfilt
@@ -76,7 +77,7 @@ class model:
     ```
     """
     mod_types                   = ['QAM', 'PSK']
-    noise_types                 = ['white', 'gray', 'pink']
+    noise_to_id                 = {'white' : '192v3dbPctSFiKM-QqzYAgd6FvZ4iffNL', 'gray' : '1LbFgpAtSFkTTx1pslI6ZbkAswLa0k5rv', 'pink': '1LDOjzZAJVEcD7UCPWohjk_QLYIX_dIiS'}
     data_samplerate             = 48000
     dsss_data_samplerate        = 96000
     modulated_data_samplerate   = 96000
@@ -93,10 +94,11 @@ class model:
     demodulated_data            = None
     recivied_data_bin           = None
     recivied_data               = None
+    workspace_path              = None
 
-    def __init__(self, data_path, mod_type, mod_order, polinom, noise_path, max_attenuation, SNRdB, sine_freq_multiplier,  nbits = 9, log = True, print_logo = False):
+    def __init__(self, data_path, mod_type, mod_order, polinom, noise_type, max_attenuation, SNRdB, sine_freq_multiplier,  nbits = 9, log = True, print_logo = False):
         if(mod_type not in self.mod_types): raise ValueError("Поддерживаемые типы модуляции: 'PSK', 'QAM'")
-        # if(noise_type not in self.noise_types): raise ValueError("Поддерживаемые типы шума: 'white', 'pink', 'gray'")
+        if(noise_type not in self.noise_to_id): raise ValueError("Поддерживаемые типы шума: 'white', 'pink', 'gray'")
         if(int(sine_freq_multiplier) > 4): raise ValueError("Частота синусоиды может быть больше максимальной частоты информационного сигнала не более, чем в 4 раза")
         if(int(sine_freq_multiplier) < 0): raise ValueError("Частота синусоиды должна быть больше максимальной частоты информационного сигнала")
         self.channel_signal_sample_rate = self.modulated_data_samplerate*sine_freq_multiplier
@@ -107,11 +109,19 @@ class model:
         self.poli                       = polinom
         self.mod_type                   = mod_type
         self.mod_order                  = mod_order
-        self.noise_path                 = noise_path
+        self.noise_type                 = noise_type
         self.max_attenuation            = max_attenuation
         self.SNRdB                      = SNRdB
         self.sine_freq_multiplier       = int(sine_freq_multiplier)
         self.print_logo                 = print_logo
+
+        if os.path.isdir('workspace') == False:
+            os.mkdir('workspace')
+            self.workspace_path = os.path.abspath('workspace')
+            if self.log: print("Создана рабочая директория workspace: ", self.workspace_path)
+        else:
+            self.workspace_path = os.path.abspath('workspace') 
+            if self.log: print("Рабочая директория workspace уже существует: ", self.workspace_path)
         
     def plot_logo(self):
         
@@ -131,6 +141,7 @@ class model:
         fig, ax = plt.subplots(1,figsize=(15,5))
         fig.suptitle(name, fontsize=16)
         ax.psd(signal, len(signal), fs)
+        plt.show()
         
     def resample(self, signal, new_rate, old_rate ):
         number_of_samples = round(len(signal) * float(new_rate) / old_rate)
@@ -138,7 +149,7 @@ class model:
 
     def export_recivied_audio(self, fname = 'recivied_audio' ):
         fname = fname + '.wav'
-        sf.write(fname, self.recivied_data, self.data_samplerate)
+        sf.write(os.path.join(self.workspace_path, fname), self.recivied_data, self.data_samplerate)
         if self.log: print("Экспорт аудиофайла:\n -Аудиофайл успешно экспортирован как", fname)
 
     def read_data(self):
@@ -199,18 +210,49 @@ class model:
         if self.log: print(' - Размерность сигнала:', self.modulated_data.shape)
         if self.log: self.plot_psd(self.modulated_data, self.modulated_data_samplerate, name= "Спектральная плотность мощности модулированного сигнала")
 
+    def download_file_from_google_drive(self, id, destination):
+        URL = "https://docs.google.com/uc?export=download"
+        session = requests.Session()
+        response = session.get(URL, params = { 'id' : id }, stream = True)
+        token = self.get_confirm_token(response)
+        if token:
+            params = { 'id' : id, 'confirm' : token }
+            response = session.get(URL, params = params, stream = True)
+        self.save_response_content(response, destination)    
+
+    def get_confirm_token(self, response):
+        for key, value in response.cookies.items():
+            if key.startswith('download_warning'):
+                return value
+        return None
+
+    def save_response_content(self, response, destination):
+        CHUNK_SIZE = 32768
+        with open(destination, "wb") as f:
+            for chunk in response.iter_content(CHUNK_SIZE):
+                if chunk: # filter out keep-alive new chunks
+                    f.write(chunk)
+                
+    def prepare_noise(self):
+        noise_path = os.path.join(self.workspace_path, self.noise_type + '.wav')
+        if (not os.path.isfile(noise_path)):
+            if self.log: print('Загрузка файла с шумом. Это может занять некоторое время...')
+            file_id = self.noise_to_id[self.noise_type]
+            self.download_file_from_google_drive(file_id, noise_path)
+
+
     def add_noise(self):
         if self.log: print('Добавление шума:')
-        if self.log: print(' - Загрузка аудиофайла с шумом')
-        filename = self.noise_path
+        self.prepare_noise()
+        filename = os.path.join(self.workspace_path, self.noise_type + '.wav')
         noise, self.noise_samplerate = sf.read(filename)
         if self.log: print(' - Размерность шума:', noise.shape)
         noise = noise[0:self.channel_signal.shape[0]]
         if self.log: print(' - Размерность шума после обрезания(кек):', noise.shape)
         P_sig = (np.mean(self.channel_signal.real**2)+np.mean(self.channel_signal.imag**2))/2
-        if self.log: print(' - Мощность сигнала равна: ', P_sig)
+        if self.log: print(' - Мощность сигнала (в среднем на один канал) равна: ', P_sig)
         P_noise = np.mean(noise**2)
-        if self.log: print(' - Мощность шума равна: ', P_noise)
+        if self.log: print(' - Мощность шума (один канал) равна: ', P_noise)
         gamma = 10**(self.SNRdB/10) #SNR to linear scale
         if self.log: print(' - SNR в разах: ', gamma)
         target_noise_amp = np.sqrt(P_sig/gamma)
@@ -219,7 +261,7 @@ class model:
         noise *= noise_scale_coeff
         self.noise = noise + noise*1j
         P_noise2 = np.mean(noise**2)
-        if self.log: print(' - Мощность шума после скалирования: ', P_noise2)
+        if self.log: print(' - Мощность шума (в среднем на один канал) после скалирования: ', P_noise2)
         if self.log: self.plot_psd(self.noise, self.noise_samplerate, name = "Спектральная плотность мощности шумового сигнала")
         if self.log: print(' - Сложение сигнала с шумом')
         self.channel_signal = self.channel_signal + self.noise
@@ -255,7 +297,7 @@ class model:
         self.add_sine()
 
     def filter_signal(self):
-        cutoff = m.treshold_freq*1.1
+        cutoff = self.treshold_freq*1.1
         if self.log: print(' - Создание LPF с частотой среза',cutoff )
         nyq = 0.5 * self.channel_signal_sample_rate
         normal_cutoff = cutoff / nyq
@@ -265,7 +307,7 @@ class model:
         w, h = freqz(b, a, worN=80000)
         ax.plot(0.5*self.channel_signal_sample_rate*w/np.pi, 20 * np.log10(abs(h)), 'b')
         ax.plot(cutoff, 0.5*np.sqrt(2), 'ko')
-        ax.axvline(m.treshold_freq, color='k')
+        ax.axvline(self.treshold_freq, color='k')
         ax.set_xlim(0, 0.5*self.channel_signal_sample_rate)
         ax.set_title("АЧХ фильтра")
         ax.set_ylabel('Амплитуда [дБ]', color='b')
